#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#######################################################################
#
# VidCutter - media cutter & joiner
#
# copyright © 2018 Pete Alexandrou
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#######################################################################

__version__ = '6.0.5.2'

__appname__ = 'VidCutter'
__domain__ = 'ozmartians.com'
__desktopid__ = 'com.ozmartians.VidCutter'
__appid__ = 'vidcutter-CCDC440A-CC57-4BED-8CDE-1DA285976A64'

__author__ = 'Pete Alexandrou'
__email__ = 'pete@ozmartians.com'
__website__ = 'https://vidcutter.ozmartians.com'
__bugreport__ = 'https://github.com/ozmartian/vidcutter/issues'

__ispypi__ = False
